﻿using Sdl.Core.Globalization;
using Sdl.FileTypeSupport.Framework;
using Sdl.FileTypeSupport.Framework.IntegrationApi;
using Sdl.FileTypeSupport.Framework.NativeApi;
using Sdl.FileTypeSupport.Framework.PreviewControls;
using Xliff20FileType.Preview;
using Xliff20FileType.Writer;
using Xliff20FileType.Sniffer;
using Xliff20FileType.Verifier;

namespace Xliff20FileType
{
    [FileTypeComponentBuilder(Id = "Xliff20_FilterComponentBuilderExtension_Id",
                                       Name = "Plugin_Nice_Name",
                                       Description = "Plugin_Description")]
    public class ComponentBuilder : IFileTypeComponentBuilder
    {
        public IFileTypeManager FileTypeManager { get; set; }

        public IFileTypeDefinition FileTypeDefinition { get; set; }

        public IFileTypeInformation BuildFileTypeInformation(string name)
        {
            IFileTypeInformation info = this.FileTypeManager.BuildFileTypeInformation();

            info.FileTypeDefinitionId = new FileTypeDefinitionId("XLIFF 2.0 File Type 1.0.0.0");
            info.FileTypeName = new LocalizableString("XLIFF 2.0");
            info.FileTypeDocumentName = new LocalizableString("XLIFF 2.0 Document");
            info.FileTypeDocumentsName = new LocalizableString("XLIFF 2.0 Documents");
            info.Description = new LocalizableString("XLIFF 2.0 documents");
            info.FileDialogWildcardExpression = "*.xlf";
            info.DefaultFileExtension = "xlf";
            info.Icon = new IconDescriptor(PluginResources.icon);
            info.Enabled = true;

            return info;
        }

        public INativeFileSniffer BuildFileSniffer(string name)
        {
            return new XliffSniffer();
        }

        public IFileExtractor BuildFileExtractor(string name)
        {
            XliffParser parser = new XliffParser();
            return FileTypeManager.BuildFileExtractor(parser, this);
        }

        public IFileGenerator BuildFileGenerator(string name)
        {
            XliffWriter writer = new XliffWriter();
            return FileTypeManager.BuildFileGenerator(writer);
        }


        public IPreviewSetsFactory BuildPreviewSetsFactory(string name)
        {
            IPreviewSetsFactory previewFactory = FileTypeManager.BuildPreviewSetsFactory();
            //Internal Preview Set
            IPreviewSet internalPreviewSet = previewFactory.CreatePreviewSet();
            internalPreviewSet.Id = new PreviewSetId("InternalPreview");
            internalPreviewSet.Name = new LocalizableString("Preview");
            internalPreviewSet.Description = new LocalizableString("Show preview");

            IControlPreviewType sourceControlPreviewType = previewFactory.CreatePreviewType<IControlPreviewType>() as IControlPreviewType;
            if (sourceControlPreviewType != null)
            {
                sourceControlPreviewType.TargetGeneratorId = new GeneratorId("HtmlPreview");
                sourceControlPreviewType.SingleFilePreviewControlId = new PreviewControlId("InternalPreview");
                internalPreviewSet.Source = sourceControlPreviewType;
            }

            IControlPreviewType targetControlPreviewType = previewFactory.CreatePreviewType<IControlPreviewType>() as IControlPreviewType;
            if (targetControlPreviewType != null)
            {
                targetControlPreviewType.TargetGeneratorId = new GeneratorId("HtmlPreview");
                targetControlPreviewType.SingleFilePreviewControlId = new PreviewControlId("InternalPreview");
                internalPreviewSet.Target = targetControlPreviewType;
            }


            //Add them all
            previewFactory.GetPreviewSets(null).Add(internalPreviewSet);
            previewFactory.GetPreviewSets(null).DefaultControlPreview = internalPreviewSet.Id;
            //previewFactory.GetPreviewSets(null).DefaultApplicationPreview = externalRealPreviewSet.Id;

            return previewFactory;
        }

        public IAbstractPreviewControl BuildPreviewControl(string name)
        {
            return new GenericInternalWebBrowserPreviewControl();
        }

        public IQuickTagsFactory BuildQuickTagsFactory(string name)
        {
            return FileTypeManager.BuildQuickTagsFactory();
        }

        public IVerifierCollection BuildVerifierCollection(string name)
        {
            IVerifierCollection verifiers = FileTypeManager.BuildVerifierCollection();
            verifiers.BilingualVerifiers.Add(new XliffVerifier());
            return verifiers;
        }

        public IAbstractGenerator BuildAbstractGenerator(string name)
        {
            return FileTypeManager.BuildFileGenerator(new HtmlPreviewWriter());
        }

        public IAdditionalGeneratorsInfo BuildAdditionalGeneratorsInfo(string name)
        {
            return null;
        }

        public IBilingualDocumentGenerator BuildBilingualGenerator(string name)
        {
            return null;
        }

        public IAbstractPreviewApplication BuildPreviewApplication(string name)
        {
            return null;
        }
    }
}