﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using Sdl.FileTypeSupport.Framework.BilingualApi;
using Sdl.FileTypeSupport.Framework.NativeApi;
using Xliff20FileType.Common;

namespace Xliff20FileType.Parser
{
    class BcmGeneratorRepository
    {
        private Dictionary<NodeSpec, AbstractBcmGenerator> _generators = new Dictionary<NodeSpec, AbstractBcmGenerator>();

        public BcmGeneratorRepository()
        {
            AddGenerator(new TextGenerator());
            AddGenerator(new PhGenerator());
            AddGenerator(new PcGenerator());
        }

        public IDocumentItemFactory ItemFactory
        {
            get;
            set;
        }

        public IPropertiesFactory PropertiesFactory
        {
            get;
            set;
        }

        private void AddGenerator(AbstractBcmGenerator generator)
        {
            generator.Repository = this;

            _generators[generator.SupportedNode] = generator;
        }

        public void GenerateContent(UnitInfo unitInfo, XNode node, IAbstractMarkupDataContainer targetContainer)
        {
            AbstractBcmGenerator generator = GetBcmGenerator(node);
            generator.GenerateContent(unitInfo, node, targetContainer);
        }

        public void GenerateContent(UnitInfo unitInfo, IEnumerable<XNode> nodes, IAbstractMarkupDataContainer targetContainer)
        {
            foreach (XNode node in nodes)
            {
                GenerateContent(unitInfo, node, targetContainer);
            }
        }

        public AbstractBcmGenerator GetBcmGenerator(XNode node)
        {
            AbstractBcmGenerator generator;

            NodeSpec nodeSpec = GetNodeSpec(node);
            if (_generators.TryGetValue(nodeSpec, out generator))
            {
                return generator;
            }

            throw new Exception(String.Format("No BCM generator found for element '{0}'.", nodeSpec));

        }

        private NodeSpec GetNodeSpec(XNode node)
        {
            return new NodeSpec { NodeType = node.NodeType, ElementName = (node is XElement) ? ((XElement)node).Name : null  };
        }
    }
}
