﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Sdl.FileTypeSupport.Framework.BilingualApi;
using System.Xml.Linq;
using Sdl.FileTypeSupport.Framework.NativeApi;
using Sdl.FileTypeSupport.Framework.Formatting;
using Xliff20FileType.Common;

namespace Xliff20FileType.Parser
{
    class PcGenerator : AbstractBcmGenerator
    {
        public override NodeSpec SupportedNode
        {
            get { return new NodeSpec {  NodeType = System.Xml.XmlNodeType.Element, ElementName = XName.Get("pc", XliffConstants.Xliff20CoreNamespace) }; }
        }

        public override void GenerateContent(UnitInfo unitInfo, XNode node, IAbstractMarkupDataContainer targetContainer)
        {
            XElement pcElement = (XElement)node;

            string dataRefStart = pcElement.Attribute("dataRefStart").Value;
            string dataRefEnd = pcElement.Attribute("dataRefEnd").Value;
            OriginalData dataStart = unitInfo.GetOriginalData(dataRefStart);
            OriginalData dataEnd = unitInfo.GetOriginalData(dataRefEnd);

            // basic properties - start and end tag
            IStartTagProperties startTagProperties  = PropertiesFactory.CreateStartTagProperties(dataStart.Contents);
            startTagProperties.TagId = new TagId(pcElement.Attribute("id").Value);
            startTagProperties.DisplayText = (string)pcElement.Attribute("dispStart") ?? dataStart.Contents;

            IEndTagProperties endTagProperties = PropertiesFactory.CreateEndTagProperties(dataEnd.Contents);
            endTagProperties.DisplayText = (string)pcElement.Attribute("dispEnd") ?? dataEnd.Contents;

            // formatting
            XAttribute fsAttribute = pcElement.Attribute(XliffConstants.fs);
            if (fsAttribute != null)
            {
                startTagProperties.SetMetaData("fs", fsAttribute.Value);

                startTagProperties.Formatting = XliffFormatting.CreateFormatting(PropertiesFactory.FormattingItemFactory, fsAttribute.Value);

                if (startTagProperties.Formatting.Count > 0)
                {
                    // formatting is visible -> allow hiding tag
                    startTagProperties.CanHide = true;
                    endTagProperties.CanHide = true;
                }
            }
            
            // validation
            XAttribute canDeleteAttribute = pcElement.Attribute("canDelete");
            if (canDeleteAttribute != null)
            {
                startTagProperties.SetMetaData("canDelete", canDeleteAttribute.Value);
            }
                       

            // create tag and add to container
            ITagPair tagPair = ItemFactory.CreateTagPair(startTagProperties, endTagProperties);
            targetContainer.Add(tagPair);

            // generate child content
            Repository.GenerateContent(unitInfo, pcElement.Nodes(), tagPair);
        }
    }
}
