﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Sdl.FileTypeSupport.Framework.Formatting;
using System.Drawing;
using Sdl.FileTypeSupport.Framework.NativeApi;
using Sdl.FileTypeSupport.Framework.Core.Utilities.NativeApi;

namespace Xliff20FileType.Parser
{
    class XliffContext
    {
        private static string[] _structureContextValues = new String[] { "h1", "h2", "h3", "h4", "h5", "h6", "div", "ul" };
        public static bool IsStructureContext(IContextInfo contextInfo)
        {
            string fsValue = contextInfo.GetMetaData("fs");
            return _structureContextValues.Contains(fsValue);
        }

        public static ContextData CreateContextInfo(IPropertiesFactory propertiesFactory, Stack<ContextData> contexts, string fsValue, bool useDisplayName)
        {
            IContextInfo contextInfo = null;
            IStructureInfo structureInfo = null;
            switch (fsValue)
            {
                case "h1":
                case "h2":
                case "h3":
                case "h4":
                case "h5":
                case "h6":
                    contextInfo = propertiesFactory.CreateContextInfo(TMMatchContextTypes.Heading);
                    contextInfo.Purpose = ContextPurpose.Match;
                    break;
                case "ul":
                    contextInfo = propertiesFactory.CreateContextInfo(StandardContextTypes.UnorderedList);
                    contextInfo.Purpose = ContextPurpose.Information;
                    break;
                case "li":
                    contextInfo = propertiesFactory.CreateContextInfo(TMMatchContextTypes.ListItem);
                    contextInfo.Purpose = ContextPurpose.Match;
                    break;
                case "a":
                    contextInfo = propertiesFactory.CreateContextInfo(StandardContextTypes.Hyperlink);
                    break;
                default:
                    // paragraph
                    contextInfo = propertiesFactory.CreateContextInfo(StandardContextTypes.Paragraph);
                    contextInfo.Purpose = ContextPurpose.Match;
                    break;
            }

            contextInfo.DefaultFormatting = XliffFormatting.CreateFormatting(propertiesFactory.FormattingItemFactory, fsValue);
            contextInfo.SetMetaData("fs", fsValue ?? "p");

            if (_structureContextValues.Contains(fsValue))
            {
                // set parent
                structureInfo = propertiesFactory.CreateStructureInfo(contextInfo, false, null);
                structureInfo.MustUseDisplayName = useDisplayName;
                structureInfo.ParentStructure = contexts.Select(cd => cd.StructureInfo).FirstOrDefault(si => si != null);
            }
            
            return new ContextData { Context = contextInfo, StructureInfo = structureInfo };
        }
    }
}
