﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Sdl.FileTypeSupport.Framework.NativeApi;
using Sdl.FileTypeSupport.Framework.BilingualApi;
using System.Xml;
using System.Xml.Linq;
using Xliff20FileType.Parser;

namespace Xliff20FileType.Preview
{
    class HtmlPreviewWriter : AbstractBilingualFileTypeComponent, IBilingualWriter, INativeOutputSettingsAware
    {
        private IPersistentFileConversionProperties _fileConversionProperties;
        private INativeOutputFileProperties _outputFileProperties;
        private XmlWriter _writer;

        private List<IStructureInfo> _openStructures = new List<IStructureInfo>();

        public void Initialize(IDocumentProperties documentInfo)
        {
           
        }

        public void SetFileProperties(IFileProperties fileInfo)
        {
            //fileInfo.FileConversionProperties.
        }

        public void ProcessParagraphUnit(IParagraphUnit paragraphUnit)
        {
            int localStructures = WriteStructures(paragraphUnit);

            IParagraph paragraph = _outputFileProperties.ContentRestriction == ContentRestriction.Source
                ? paragraphUnit.Source : paragraphUnit.Target;

            new ParagraphHtmlPreviewWriter(_writer).VisitParagraph(paragraph);

            // close local structures
            for (int i = 0; i < localStructures; i++)
            {
                _writer.WriteEndElement();
            }
        }

        private int WriteStructures(IParagraphUnit paragraphUnit)
        {
            // close al structures that need closing
            List<IStructureInfo> puStructures = GetStructures(paragraphUnit);

            int index = 0;
            while (index < puStructures.Count && index < _openStructures.Count && _openStructures[index] == puStructures[index])
            {
                index++;
            }

            // close open structures first
            while (index < _openStructures.Count)
            {
                _writer.WriteEndElement();
                _openStructures.RemoveAt(_openStructures.Count - 1);
            }

            // now open structures that need opening
            while (index < puStructures.Count)
            {
                string fsValue = puStructures[index].ContextInfo.GetMetaData("fs");
                _writer.WriteStartElement(fsValue);
                _openStructures.Add(puStructures[index]);
                index++;
            }

            // now open local paragraph contexts that are not structural 
            IStructureInfo si = paragraphUnit.Properties.Contexts.StructureInfo;
            IList<IContextInfo> contexts = paragraphUnit.Properties.Contexts.Contexts;
            int localContextCount = 0;
            while (localContextCount < contexts.Count && !XliffContext.IsStructureContext(contexts[localContextCount]))
            {
                localContextCount++;
            }

            // write in oposite order
            for (int i = localContextCount - 1; i >= 0; i--)
            {
                string fsValue = contexts[i].GetMetaData("fs");
                _writer.WriteStartElement(fsValue);
                
            }
            
            return localContextCount;
        }

        private List<IStructureInfo> GetStructures(IParagraphUnit pu)
        {
            List<IStructureInfo> structures = new List<IStructureInfo>();

            IStructureInfo si = pu.Properties.Contexts.StructureInfo;
            while (si != null)
            {
                structures.Insert(0, si);
                si = si.ParentStructure;
            }

            return structures;
        }

        public void FileComplete()
        {
        }

        public void Complete()
        {
            // close open structures first
            while (_openStructures.Count > 0)
            {
                _writer.WriteEndElement();
                _openStructures.RemoveAt(_openStructures.Count-1);
            }

            _writer.WriteEndElement(); // body
            _writer.WriteEndElement(); // html
            _writer.Close();
        }

        public void GetProposedOutputFileInfo(IPersistentFileConversionProperties fileProperties, IOutputFileInfo proposedFileInfo)
        {
            _fileConversionProperties = fileProperties;

            proposedFileInfo.Filename = Guid.NewGuid() + ".html";
            
        }

        public void SetOutputProperties(INativeOutputFileProperties properties)
        {
            _outputFileProperties = properties;
            _writer = XmlWriter.Create(_outputFileProperties.OutputFilePath);

            _writer.WriteStartElement("html");

            _writer.WriteStartElement("head");
            _writer.WriteStartElement("title");
            _writer.WriteString("Preview");
            _writer.WriteEndElement();
            _writer.WriteEndElement();

            _writer.WriteStartElement("body");           
        }

        public void Dispose()
        {
        }
    }
}
