﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Sdl.FileTypeSupport.Framework.BilingualApi;
using System.Xml;
using Sdl.FileTypeSupport.Framework.NativeApi;

namespace Xliff20FileType.Preview
{
    class ParagraphHtmlPreviewWriter : IMarkupDataVisitor
    {
        private XmlWriter _writer;

        public ParagraphHtmlPreviewWriter(XmlWriter writer)
        {
            _writer = writer;
        }

        public void VisitParagraph(IParagraph paragraph)
        {
            VisitChildren(paragraph);
        }

        public void VisitChildren(IAbstractMarkupDataContainer container)
        {
            foreach (var item in container)
            {
                item.AcceptVisitor(this);
            }
        }

        public void VisitCommentMarker(ICommentMarker commentMarker)
        {
            VisitChildren(commentMarker);
        }

        public void VisitLocationMarker(ILocationMarker location)
        {
            
        }

        public void VisitLockedContent(ILockedContent lockedContent)
        {
            VisitChildren(lockedContent.Content);
        }

        public void VisitOtherMarker(IOtherMarker marker)
        {
            VisitChildren(marker);
        }

        public void VisitPlaceholderTag(IPlaceholderTag tag)
        {
            //VisitChildren(tag);
        }

        public void VisitRevisionMarker(IRevisionMarker revisionMarker)
        {
            throw new NotImplementedException();
        }

        public void VisitSegment(ISegment segment)
        {
            VisitChildren(segment);
        }

        public void VisitTagPair(ITagPair tagPair)
        {
            bool element = WriteHtmlElement(tagPair.StartTagProperties);
            VisitChildren(tagPair);
            if (element)
            {
                _writer.WriteEndElement();
            }
        }

        public void VisitText(IText text)
        {
            _writer.WriteString(text.Properties.Text);
        }

        private bool WriteHtmlElement(IMetaDataContainer container)
        {
            string fsValue = container.GetMetaData("fs");
            if (fsValue != null)
            {
                _writer.WriteStartElement(fsValue);
                return true;
            }
            else
            {
                return false;
            }
        }
    }
}
