﻿using Sdl.Core.Globalization;
using Sdl.Core.Settings;
using Sdl.FileTypeSupport.Framework.NativeApi;
using System.Xml;
using Sdl.FileTypeSupport.Framework;
using Xliff20FileType.Common;

namespace Xliff20FileType.Sniffer
{
    public class XliffSniffer : INativeFileSniffer
    {
        public SniffInfo Sniff(string nativeFilePath, Language suggestedSourceLanguage, Codepage suggestedCodepage,
                               INativeTextLocationMessageReporter messageReporter, ISettingsGroup settingsGroup)
        {
            try
            {
                using (XmlReader reader = XmlReader.Create(nativeFilePath))
                {
                    XmlNodeType nodeType = reader.MoveToContent();
                    if (reader.Name != "xliff")
                    {
                        return new SniffInfo { IsSupported = false };
                    }

                    if (!reader.NamespaceURI.Equals(XliffConstants.Xliff20CoreNamespace, System.StringComparison.OrdinalIgnoreCase))
                    {
                        return new SniffInfo { IsSupported = false };
                    }

                    string sourceLanguage = reader.GetAttribute("srcLang");
                    if (string.IsNullOrEmpty(sourceLanguage))
                    {
                        return new SniffInfo { IsSupported = false }; 
                    }

                    string targetLanguage = reader.GetAttribute("tgtLang");


                    return new SniffInfo 
                    { 
                        IsSupported = true, 
                        DetectedSourceLanguage = new Pair<Language, DetectionLevel>(new Language(sourceLanguage), DetectionLevel.Certain), 
                        DetectedTargetLanguage = (string.IsNullOrEmpty(targetLanguage) 
                            ? null : 
                            new Pair<Language, DetectionLevel>(new Language(targetLanguage), DetectionLevel.Certain)) };
                }
            }
            catch
            {
                return new SniffInfo { IsSupported = false };
            }
        }
    }
}
