﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Sdl.FileTypeSupport.Framework.BilingualApi;
using Sdl.FileTypeSupport.Framework.NativeApi;

namespace Xliff20FileType.Verifier
{
    class XliffVerifier : IBilingualVerifier
    {
        #region Setup
        public IDocumentItemFactory ItemFactory
        {
            get;
            set;
        }

        public IBilingualContentMessageReporter MessageReporter
        {
            get;
            set;
        }

        public void Initialize(IDocumentProperties documentInfo)
        {
        }

        public void SetFileProperties(IFileProperties fileInfo)
        {
        }

        public void FileComplete()
        {
        }


        public void Complete()
        {
        }

        #endregion Setup

        #region Verification

        public void ProcessParagraphUnit(IParagraphUnit paragraphUnit)
        {
            foreach (ISegmentPair segmentPair in paragraphUnit.SegmentPairs)
            {
                CheckCanDelete(segmentPair);
            }
        }

        private void CheckCanDelete(ISegmentPair segmentPair)
        {
            foreach (IPlaceholderTag sourceTag in segmentPair.Source.AllSubItems.OfType<IPlaceholderTag>().Where(ph =>
                ph.Properties.GetMetaData("canDelete") == "no"))
            {
                if (segmentPair.Target.AllSubItems.OfType<IPlaceholderTag>().FirstOrDefault(ph =>
                    ph.Properties.TagId.Id == sourceTag.Properties.TagId.Id) == null)
                {
                    Location tagLocation = new Location(sourceTag.Parent, sourceTag);
                    Location tagEndLocation = new Location(sourceTag.Parent, sourceTag);
                    tagEndLocation.MoveNextSibling();
                    MessageReporter.ReportMessage(
                        this, 
                        "XLIFF 2.0", 
                        ErrorLevel.Error, 
                        "Tag should not be deleted", 
                        new TextLocation(tagLocation, 0), 
                        new TextLocation(tagEndLocation, 0));
                }
            }
        }

        #endregion Verification


    }
}
