﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Sdl.FileTypeSupport.Framework.BilingualApi;
using System.Xml.Linq;
using Xliff20FileType.Common;

namespace Xliff20FileType.Writer
{
    class XliffUnitUpdater : IMarkupDataVisitor
    {
        private UnitInfo _unitInfo;
        private XElement _targetElement;

        public XliffUnitUpdater(UnitInfo unitInfo, XElement targetElement)
        {
            _unitInfo = unitInfo;
            _targetElement = targetElement;
        }

        public void VisitChildren(IAbstractMarkupDataContainer container)
        {
            foreach (var item in container)
            {
                item.AcceptVisitor(this);
            }
        }


        public void VisitCommentMarker(ICommentMarker commentMarker)
        {
            
        }

        public void VisitLocationMarker(ILocationMarker location)
        {
        }

        public void VisitLockedContent(ILockedContent lockedContent)
        {
        }

        public void VisitOtherMarker(IOtherMarker marker)
        {
        }

        public void VisitPlaceholderTag(IPlaceholderTag tag)
        {
            XElement phElement = new XElement(XliffConstants.ph);
            phElement.SetAttributeValue("id", tag.TagProperties.TagId.Id);
            phElement.SetAttributeValue("disp", tag.TagProperties.DisplayText);
            
            string canDelete = tag.TagProperties.GetMetaData("canDelete");
            if (canDelete != null)
            {
                phElement.SetAttributeValue("canDelete", canDelete);
            }

            OriginalData data = _unitInfo.AddOriginalData(tag.TagProperties.TagContent);
            phElement.SetAttributeValue("dataRef", data.Id);

            
            _targetElement.Add(phElement);
        }

        public void VisitRevisionMarker(IRevisionMarker revisionMarker)
        {
        }

        public void VisitSegment(ISegment segment)
        {
            VisitChildren(segment);
        }

        public void VisitTagPair(ITagPair tagPair)
        {
            XElement pcElement = new XElement(XliffConstants.pc);
            pcElement.SetAttributeValue("id", tagPair.StartTagProperties.TagId.Id);
            pcElement.SetAttributeValue("dispStart", tagPair.StartTagProperties.DisplayText);
            pcElement.SetAttributeValue("dispEnd", tagPair.EndTagProperties.DisplayText);

            string canDelete = tagPair.StartTagProperties.GetMetaData("canDelete");
            if (canDelete != null)
            {
                pcElement.SetAttributeValue("canDelete", canDelete);
            }

            string fs = tagPair.StartTagProperties.GetMetaData("fs");
            if (fs != null)
            {
                pcElement.SetAttributeValue(XliffConstants.fs, fs);
            }

            OriginalData startData = _unitInfo.AddOriginalData(tagPair.StartTagProperties.TagContent);
            pcElement.SetAttributeValue("dataRefStart", startData.Id);
            
            OriginalData endData = _unitInfo.AddOriginalData(tagPair.EndTagProperties.TagContent);
            pcElement.SetAttributeValue("dataRefEnd", endData.Id);

            _targetElement.Add(pcElement);

            new XliffUnitUpdater(_unitInfo, pcElement).VisitChildren(tagPair);
        }

        public void VisitText(IText text)
        {
            _targetElement.Add(text.Properties.Text);
        }
    }
}
