﻿using Sdl.Core.Settings;
using Sdl.FileTypeSupport.Framework.BilingualApi;
using Sdl.FileTypeSupport.Framework.IntegrationApi;
using Sdl.FileTypeSupport.Framework.NativeApi;
using System.Linq;
using System.Xml.Linq;
using System.Xml.XPath;
using System.Xml;
using Xliff20FileType.Writer;
using System.Collections.Generic;
using Xliff20FileType.Common;

namespace Xliff20FileType.Writer
{
    class XliffWriter : AbstractBilingualFileTypeComponent, IBilingualWriter, INativeOutputSettingsAware
    {
        private IFileProperties _fileProperties;
        private IPersistentFileConversionProperties _fileConversionProperties;
        private INativeOutputFileProperties _outputFileProperties;
        private XDocument _doc;
        XmlNamespaceManager _nsMgr;

        public void Initialize(IDocumentProperties documentInfo)
        {
            
        }

        public void Complete()
        {
            _doc.Save(_outputFileProperties.OutputFilePath);
        }

        public void SetFileProperties(IFileProperties fileInfo)
        {
            _fileProperties = fileInfo;
            string xliff20FilePath = _fileProperties.FileConversionProperties.InputFilePath;
            _doc = XDocument.Load(xliff20FilePath, LoadOptions.PreserveWhitespace);
            _nsMgr = new XmlNamespaceManager(new NameTable());
            _nsMgr.AddNamespace("xliff", XliffConstants.Xliff20CoreNamespace);
            _nsMgr.AddNamespace("fs", XliffConstants.Xliff20FormatStyleNamespace);
        }

        public void FileComplete()
        {
        }

        public void ProcessParagraphUnit(IParagraphUnit paragraphUnit)
        {
            // find corresponding unit element
            XElement unitElement = _doc.XPathSelectElement(string.Format("//xliff:unit[@id='{0}']", paragraphUnit.Properties.ParagraphUnitId.Id), _nsMgr);

            XElement originalDataElement = unitElement.Element(XliffConstants.originalData);

            UnitInfo unitInfo = new UnitInfo { UnitElement = unitElement, OriginalData = ParseOriginalData(originalDataElement) };

            // ignore content between segments for now
            foreach (ISegmentPair segmentPair in paragraphUnit.SegmentPairs)
            {
                // find segment pair in xliff
                XElement segmentElement = unitElement.XPathSelectElement(string.Format("xliff:segment[@id='{0}']", segmentPair.Properties.Id.Id), _nsMgr);

                // replace content
                XElement targetElement = segmentElement.Element(XliffConstants.target);
                targetElement.RemoveNodes();
                XliffUnitUpdater unitUpdater = new XliffUnitUpdater(unitInfo, targetElement);
                unitUpdater.VisitSegment(segmentPair.Target);
            }

            // replace original data
            if (originalDataElement == null && unitInfo.OriginalData.Count > 0)
            {
                originalDataElement = new XElement(XliffConstants.originalData);
                unitElement.Add(originalDataElement);
            }

            if (originalDataElement != null)
            {
                originalDataElement.RemoveNodes();

                foreach (OriginalData originalData in unitInfo.OriginalData)
                {
                    originalDataElement.Add(new XElement(XliffConstants.data, new XAttribute("id", originalData.Id), originalData.Contents));
                }
            }
        }

        private List<OriginalData> ParseOriginalData(XElement originalDataElement)
        {
            if (originalDataElement == null)
            {
                return new List<OriginalData>();
            }
            
            return (from e in originalDataElement.Elements(XliffConstants.data)
                    select new OriginalData { Id = e.Attribute("id").Value, Contents = e.Value }).ToList();
        }

        public void Dispose()
        {
        }

        public void SetOutputProperties(INativeOutputFileProperties properties)
        {
            _outputFileProperties = properties;
        }

        public void GetProposedOutputFileInfo(IPersistentFileConversionProperties fileProperties, IOutputFileInfo proposedFileInfo)
        {
            _fileConversionProperties = fileProperties;
        }
    }
}
